# -*- coding: utf-8 -*-

from django.utils.translation import ugettext as _

class ModelObjectCopier():

    def __init__(
        self,
        base_object,
        related_parent_name=None,
        kwargs_excludes=['date_create', 'date_last_modify', 'status', 'ordering', 'creator', 'last_modifier'],
        related_managers_excludes=[],
        duplicate_prefix="COPY",
        as_unique_fields=[],
        dest_model=None,
    ):
        """
        This util class allows you to clone any model instance including all its related managers.

        :arg base_object: The model object instance you want to clone
        :arg related_parent_name: Specifies the name related objects are referenced to base_object
        :arg kwargs_excludes: A list of fields to exclude while creating the fields dict for new object
        :arg related_managers_excludes: A list of related managers name you want to don't clone in copy_related method
        :arg duplicate_prefix: A prefix copier appends to unique fields to don't get integrity exceptions while saving new object
        :arg as_unique_fields: A list of fields name to check as if they were unique fields
        """
        self.base_object = base_object
        self.related_parent_name = related_parent_name
        self.kwargs_excludes = kwargs_excludes
        self.related_managers_excludes = related_managers_excludes
        self.duplicate_prefix = duplicate_prefix
        self.new_object = None
        self.as_unique_fields = as_unique_fields
        self.dest_model = dest_model
        self.base_object_related_managers = [
            rel.get_accessor_name() for rel in type(self.base_object)._meta.get_all_related_objects()
            if rel.get_accessor_name() not in self.related_managers_excludes
        ]

    def _set_new_main_object(self, new_object):
        self.new_object = new_object

    def bulk_copy(self):
        """
        Allow you to copy the main object and all its related managers
        """
        try:
            self.copy_main()
            self.copy_related()
            return True, _("Clonation done"), self.new_object
        except Exception, e:
            return False, _("Error in clonation for %s: %s" % (self.base_object, e)), None

    def get_kwargs_copy(self, row):
        """
        This function creates and return a dict containing all valorized new object fields.

        :arg row: The model instance you need to copy and used to create the new one
        """
        new_kwargs = dict([
            (fld.name, getattr(row, fld.name)) for fld in row._meta.fields
            if fld.name != "id" and fld.name not in self.kwargs_excludes
        ])
        unique_fields = [field for field in type(row)._meta.fields if field._unique]
        if row == self.base_object:
            for field in self.as_unique_fields:
                unique_fields.append(type(self.base_object)._meta.get_field_by_name(field)[0])
        for field in unique_fields:

            if field.name in self.kwargs_excludes:
                continue

            value = "%s-%s" % (self.duplicate_prefix, new_kwargs[field.name])
            found_count = type(row).objects.filter(**{field.name: value}).count()
            if not found_count:
                new_kwargs[field.name] = value
            else:
                while True:
                    app_value = "%s-%s" % (value, str(found_count+1))
                    if type(row).objects.filter(**{field.name: app_value}).count():
                        found_count += 1
                        continue
                    else:
                        new_kwargs[field.name] = app_value
                        break
        return new_kwargs

    def copy_main(self):
        """
        This function allow you to copy only main fields (assumed as a header) of the main object
        """
        try:
            new_kwargs = self.get_kwargs_copy(self.base_object)
            dest_model = type(self.base_object) if self.dest_model is None else self.dest_model
            new_object = dest_model.objects.create(**new_kwargs)
            new_object.save()
            self._set_new_main_object(new_object)
            return True, _("Clonation done"), new_object
        except Exception, e:
            return False, _("Error in clonation for %s: %s" % (self.base_object, e)), None

    def copy_related(self):
        """
        This function allows you to copy all related managers of the main object.
        """
        if self.new_object and self.related_parent_name:
            for related_manager in self.base_object_related_managers:
                for related_manager_row in getattr(self.base_object, related_manager).all():
                    new_kwargs = self.get_kwargs_copy(related_manager_row)
                    new_kwargs[self.related_parent_name] = self.new_object
                    type(related_manager_row).objects.create(**new_kwargs)
